Type: Package
Package: paradox
Title: Define and Work with Parameter Spaces for Complex Algorithms
Version: 1.0.1
Authors@R: 
    c(person(given = "Michel",
             family = "Lang",
             role = "aut",
             email = "michellang@gmail.com",
             comment = c(ORCID = "0000-0001-9754-0393")),
      person(given = "Bernd",
             family = "Bischl",
             role = "aut",
             email = "bernd_bischl@gmx.net",
             comment = c(ORCID = "0000-0001-6002-6980")),
      person(given = "Jakob",
             family = "Richter",
             role = "aut",
             email = "jakob1richter@gmail.com",
             comment = c(ORCID = "0000-0003-4481-5554")),
      person(given = "Xudong",
             family = "Sun",
             role = "aut",
             email = "smilesun.east@gmail.com",
             comment = c(ORCID = "0000-0003-3269-2307")),
      person(given = "Martin",
             family = "Binder",
             role = c("aut", "cre"), 
             email = "mlr.developer@mb706.com"),
      person(given = "Marc",
             family = "Becker",
             role = "ctb",
             email = "marcbecker@posteo.de",
             comment = c(ORCID = "0000-0002-8115-0400")))
Description: Define parameter spaces, constraints and
    dependencies for arbitrary algorithms, to program on such spaces. Also
    includes statistical designs and random samplers. Objects are
    implemented as 'R6' classes.
License: LGPL-3
URL: https://paradox.mlr-org.com, https://github.com/mlr-org/paradox
BugReports: https://github.com/mlr-org/paradox/issues
Imports: backports, checkmate, data.table, methods, mlr3misc (>=
        0.9.4), R6
Suggests: rmarkdown, mlr3learners, e1071, knitr, lhs, spacefillr,
        testthat
Encoding: UTF-8
Config/testthat/edition: 3
Config/testthat/parallel: false
NeedsCompilation: no
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Collate: 'Condition.R' 'Design.R' 'Domain.R' 'Domain_methods.R'
        'NoDefault.R' 'ParamDbl.R' 'ParamFct.R' 'ParamInt.R'
        'ParamLgl.R' 'ParamSet.R' 'ParamSetCollection.R' 'ParamUty.R'
        'Sampler.R' 'Sampler1D.R' 'SamplerHierarchical.R'
        'SamplerJointIndep.R' 'SamplerUnif.R' 'asserts.R'
        'default_values.R' 'generate_design_grid.R'
        'generate_design_lhs.R' 'generate_design_random.R'
        'generate_design_sobol.R' 'helper.R' 'ps.R' 'ps_replicate.R'
        'ps_union.R' 'reexports.R' 'to_tune.R' 'zzz.R'
Packaged: 2024-07-09 11:25:23 UTC; user
Author: Michel Lang [aut] (<https://orcid.org/0000-0001-9754-0393>),
  Bernd Bischl [aut] (<https://orcid.org/0000-0001-6002-6980>),
  Jakob Richter [aut] (<https://orcid.org/0000-0003-4481-5554>),
  Xudong Sun [aut] (<https://orcid.org/0000-0003-3269-2307>),
  Martin Binder [aut, cre],
  Marc Becker [ctb] (<https://orcid.org/0000-0002-8115-0400>)
Maintainer: Martin Binder <mlr.developer@mb706.com>
Repository: CRAN
Date/Publication: 2024-07-09 13:10:02 UTC
