# Generated by roxygen2: do not edit by hand

S3method("$",Constructor)
S3method(as.data.table,ParamSet)
S3method(c,ParamSet)
S3method(condition_as_string,Condition)
S3method(condition_test,CondAnyOf)
S3method(condition_test,CondEqual)
S3method(default_values,ParamSet)
S3method(domain_check,ParamDbl)
S3method(domain_check,ParamFct)
S3method(domain_check,ParamInt)
S3method(domain_check,ParamLgl)
S3method(domain_check,ParamUty)
S3method(domain_is_bounded,Domain)
S3method(domain_is_bounded,ParamDbl)
S3method(domain_is_bounded,ParamFct)
S3method(domain_is_bounded,ParamInt)
S3method(domain_is_bounded,ParamLgl)
S3method(domain_is_bounded,ParamUty)
S3method(domain_is_categ,Domain)
S3method(domain_is_categ,ParamDbl)
S3method(domain_is_categ,ParamFct)
S3method(domain_is_categ,ParamInt)
S3method(domain_is_categ,ParamLgl)
S3method(domain_is_categ,ParamUty)
S3method(domain_is_number,Domain)
S3method(domain_is_number,ParamDbl)
S3method(domain_is_number,ParamFct)
S3method(domain_is_number,ParamInt)
S3method(domain_is_number,ParamLgl)
S3method(domain_is_number,ParamUty)
S3method(domain_nlevels,Domain)
S3method(domain_nlevels,ParamDbl)
S3method(domain_nlevels,ParamFct)
S3method(domain_nlevels,ParamInt)
S3method(domain_nlevels,ParamLgl)
S3method(domain_nlevels,ParamUty)
S3method(domain_qunif,Domain)
S3method(domain_qunif,ParamDbl)
S3method(domain_qunif,ParamFct)
S3method(domain_qunif,ParamInt)
S3method(domain_qunif,ParamLgl)
S3method(domain_qunif,ParamUty)
S3method(domain_sanitize,Domain)
S3method(domain_sanitize,ParamDbl)
S3method(domain_sanitize,ParamInt)
S3method(format,Condition)
S3method(print,Condition)
S3method(print,Domain)
S3method(print,FullTuneToken)
S3method(print,InternalTuneToken)
S3method(print,ObjectTuneToken)
S3method(print,RangeTuneToken)
S3method(rd_info,ParamSet)
export(CondAnyOf)
export(CondEqual)
export(Condition)
export(Design)
export(NO_DEF)
export(ParamSet)
export(ParamSetCollection)
export(Sampler)
export(Sampler1D)
export(Sampler1DCateg)
export(Sampler1DNormal)
export(Sampler1DRfun)
export(Sampler1DUnif)
export(SamplerHierarchical)
export(SamplerJointIndep)
export(SamplerUnif)
export(as.data.table)
export(assert_param_set)
export(condition_as_string)
export(default_values)
export(domain_assert)
export(domain_check)
export(domain_is_bounded)
export(domain_is_categ)
export(domain_is_number)
export(domain_nlevels)
export(domain_qunif)
export(domain_sanitize)
export(domain_test)
export(generate_design_grid)
export(generate_design_lhs)
export(generate_design_random)
export(generate_design_sobol)
export(p_dbl)
export(p_fct)
export(p_int)
export(p_lgl)
export(p_uty)
export(ps)
export(ps_replicate)
export(ps_union)
export(psc)
export(to_tune)
import(checkmate)
import(data.table)
import(mlr3misc)
importFrom(R6,R6Class)
importFrom(R6,is.R6Class)
importFrom(data.table,as.data.table)
importFrom(methods,is)
importFrom(stats,rnorm)
importFrom(stats,runif)
