% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Domain_methods.R
\name{domain_is_categ}
\alias{domain_is_categ}
\title{Whether a Given Domain is Categorical}
\usage{
domain_is_categ(param)
}
\arguments{
\item{param}{(\code{Domain}).}
}
\value{
\code{logical}.
}
\description{
This should generally be \code{TRUE} for categorical \code{\link{Domain}}s, such as \code{\link[=p_fct]{p_fct()}} or \code{\link[=p_lgl]{p_lgl()}}, and \code{FALSE} otherwise.
}
\keyword{internal}
