% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Domain_methods.R
\name{domain_sanitize}
\alias{domain_sanitize}
\title{Map to Acceptable Value}
\usage{
domain_sanitize(param, values)
}
\arguments{
\item{param}{(\code{Domain}).}

\item{values}{(\code{any}) -- format depending on the \code{Domain}.}
}
\value{
\code{any} -- format depending on the \code{Domain}.
}
\description{
Map values that are close enough to the given \code{\link{Domain}} to values that are truly acceptable.

This is used to map \code{numeric()} values that are close to but outside the acceptable interval to the interval bounds.
It is also used to convert integer-valued \code{numeric} values to \code{integer} values for \code{\link[=p_int]{p_int()}}.
}
\keyword{internal}
