% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Domain_methods.R
\name{domain_qunif}
\alias{domain_qunif}
\title{Transform a Numeric Value to a Sample}
\usage{
domain_qunif(param, x)
}
\arguments{
\item{param}{(\code{Domain}).}

\item{x}{\code{numeric} between 0 and 1.}
}
\value{
\code{any} -- format depending on the \code{Domain}.
}
\description{
Return a valid sample from the given \code{\link{Domain}}, given a value from the interval \verb{[0, 1]}.
}
\keyword{internal}
