% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/default_values.R
\name{default_values}
\alias{default_values}
\alias{default_values.ParamSet}
\title{Extract Parameter Default Values}
\usage{
default_values(x, ...)

\method{default_values}{ParamSet}(x, ...)
}
\arguments{
\item{x}{(\code{any})\cr
Object to extract default values from.}

\item{...}{(any)\cr
Additional arguments.}
}
\value{
\code{list()}.
}
\description{
Extract parameter default values.
}
