% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ps.R
\name{psc}
\alias{psc}
\title{Create a ParamSet Collection}
\usage{
psc(...)
}
\arguments{
\item{...}{(any)\cr
The \code{\link{ParamSet}}s from which to create the collection.}
}
\description{
Creates a \code{\link{ParamSetCollection}}.
}
