% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Domain_methods.R
\name{domain_is_bounded}
\alias{domain_is_bounded}
\title{Whether a Given Domain is Bounded}
\usage{
domain_is_bounded(param)
}
\arguments{
\item{param}{(\code{Domain}).}
}
\value{
\code{logical}.
}
\description{
This should generally be \code{TRUE} when \code{lower} and \code{upper} are given and finite, or when the \code{nlevels} is finite, and \code{FALSE} otherwise.
}
\keyword{internal}
