% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NoDefault.R
\name{NO_DEF}
\alias{NO_DEF}
\alias{NoDefault}
\alias{is_nodefault}
\title{Extra data type for "no default value"}
\description{
Special new data type for no-default.
Not often needed by the end-user, mainly internal.
\itemize{
\item \code{NO_DEF}: Singleton object for type, used in \code{\link{Domain}} when no default is given.
\item \code{is_nodefault()}: Is an object the 'no default' object?
}
}
