% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Domain_methods.R
\name{domain_nlevels}
\alias{domain_nlevels}
\title{The Number of Levels of a Given Domain}
\usage{
domain_nlevels(param)
}
\arguments{
\item{param}{(\code{Domain}).}
}
\value{
\code{numeric}.
}
\description{
This should be the number of discrete possible levels for discrete type \code{\link{Domain}}s such as \code{\link[=p_int]{p_int()}} or \code{\link[=p_fct]{p_fct()}}, and
\code{Inf} for continuous or untyped parameters.
}
\keyword{internal}
