% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asserts.R
\name{assert_param_set}
\alias{assert_param_set}
\title{Assertions for Params and ParamSets}
\usage{
assert_param_set(
  param_set,
  cl = NULL,
  no_untyped = FALSE,
  must_bounded = FALSE,
  no_deps = FALSE
)
}
\arguments{
\item{param_set}{(\code{\link{ParamSet}}).}

\item{cl}{(\code{character()})\cr
Allowed subclasses.}

\item{no_untyped}{(\code{logical(1)})\cr
Are untyped \code{\link{Domain}}s allowed?}

\item{must_bounded}{(\code{logical(1)})\cr
Only bounded \code{\link{Domain}}s allowed?}

\item{no_deps}{(\code{logical(1)})\cr
Are dependencies allowed?}
}
\value{
The checked object, invisibly.
}
\description{
Assertions for Params and ParamSets
}
