% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Domain_methods.R
\name{domain_is_number}
\alias{domain_is_number}
\title{Whether a Given Domain is Numeric}
\usage{
domain_is_number(param)
}
\arguments{
\item{param}{(\code{Domain}).}
}
\value{
\code{logical}.
}
\description{
This should generally be \code{TRUE} for discrete or continuous numeric \code{\link{Domain}}s, and \code{FALSE} otherwise.
}
\keyword{internal}
